#!/bin/sh

# RCSID $Id: vpninfo.sh,v 1.8 2003/05/21 06:04:46 rhowe Exp $

#     Copyright (C) 2003	Russell G. Howe, Atahualpa Ledesma
# 
#     This program is free software; you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation; either version 2 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program; if not, write to the Free Software
#     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Exit codes
EUSAGE=1	# Invalid parameters			- maybe this script is old?
ENOMETHOD=40	# The method requested is not supported - maybe this script is old?
ENOATTR=41	# The attribute requested is unknown    - maybe this script is old?
EBADIP=42	# The IPv4 address specified for the "tcp" method appears invalid
ENOCONN=43	# Unable to connect to the the specified IP address using the "tcp" method
ENONC=50	# tcp method chosen, but netcat not found (looks for "nc")

# Location of the data file for the "file" method
DB="/etc/ipsec.d/vpn-info.txt" # This file can be downloaded periodically?

# TCP port to use for TCP method
PORT=57575

until [ -z "$1" ]; do
	case "$1" in
		"--method")
			if [ "$METHOD" ]; then
				echo "$0: Can only specify one method at a time"
				exit $EUSAGE
			fi
			shift
			METHOD="$1"
			;;
		"--attribute")
			if [ "$ATTR" ]; then
				echo "$0: Can only specify one attribute to retrieve at a time"
				exit $EUSAGE
			fi
			shift
			ATTR="$1"
			;;
		"--ipaddr")
			if [ "$METHOD" != "tcp" ]; then
				echo "$0: --ipaddr option is only valid for the \"tcp\" method"
				exit $EUSAGE
			fi
			shift
			IPADDR="$1"
			;;
		"--id")
			if [ "$METHOD" != "file" ]; then
				echo "$0: --id option is only valid for the \"file\" method"
				exit $EUSAGE
			fi
			shift
			ID="$1"
			;;
		--help)
			echo "Usage: $0 --attribute <subnet> --method <method> [method-specific options]"
			exit 0;
			;;
		*)
			echo "$0: Unknown option \"$1\""
			echo "Usage: $0 --attribute <subnet> --method <method> [method-specific options]"
			exit $EUSAGE
			;;
	esac
	shift
done

if [ -z "$METHOD" ]; then
	echo "$0: No method specified. Use --method option."
	echo "Available methods: file tcp"
	exit $EUSAGE
else
	# If a method has certain required options, check for their presence here.
	case "$METHOD" in
		"file")
			if [ -z "$ID" ]; then
				echo "$0: \"file\" method required an --id argument"
				exit $EUSAGE
			fi
			;;
		"tcp")
			exit 0
			if [ -z "$IPADDR" ]; then
				echo "$0: tcp method requires an --ipaddr argument"
				exit $EUSAGE
			else
				# Check $IPADDR looks like an IP address
				#if [ "${IPADDR/[0-9]*.[0-9]*.[0-9]*.[0-9]*/}" ]; then
				#	echo "$0: \"$IPADDR\" does not seem to be a valid IPv4 address"
				#	exit $EBADIP
				#fi
			fi

			NC="$(which nc)"
			
			if [ -z "$NC" -a ! -x "$(which nc)" ]; then
				echo "$0: Need netcat installed for \"tcp\" method"
				exit $ENONC
			fi
			;;
	esac
fi

if [ -z "$ATTR" ]; then
	echo "$0: No attribute specified. What do you want me to find out?"
	echo "Use the --attribute option. Available attributes: STATIC_ROUTE GRE_ENDPOINT"
	exit $EUSAGE
else
	case "$ATTR" in
	"STATIC_ROUTE"|"GRE_ENDPOINT")
		;;
	*)
		echo "$0: Unknown attribute \"$ATTR\". Do I need upgrading?"
		exit $ENOATTR
		;;
	esac
fi

case "$METHOD" in
	file)
		egrep "^.*:.*:\"$ID\"\$" "$DB" | (
			IFS=":" read PEER SUBNET PEERID;
	
			case "$ATTR" in
			"STATIC_ROUTE")
				[ "$SUBNET" ] && echo "$SUBNET"
				;;
			"GRE_ENDPOINT")
				[ "$PEER" ] && echo "$PEER"
				;;
			*)
				exit $ENOATTR # Unsupported attribute
				;;
			esac

		)
		;;
	tcp)
	exit 0
		INFO="$("$NC" -w 5 "$IPADDR" "$PORT")"
		NCRET="$?"

		if [ "$NCRET" != "0" ]; then
			# nc failed
			echo "$0: $NC failed to connect to $IPADDR:$PORT (tcp). Exit code $NCRET"
			exit $ENOCONN
		fi
		
		INFO="$(echo "$INFO"|grep "$ATTR"|sed -e 's/^.*://g')"
		[ "$INFO" ] && echo "$INFO"
		exit 0
		;;
esac
